@@ -1,7 +1,5 @@
 from __future__ import annotations
 
-import sys
-
 from pathlib import Path
 from typing import TYPE_CHECKING
 from typing import Any
@@ -42,16 +40,20 @@ class InitCommand(Command):
         option(
             "dependency",
             None,
-            "Package to require, with an optional version constraint, "
-            "e.g. requests:^2.10.0 or requests=2.11.1.",
+            (
+                "Package to require, with an optional version constraint, "
+                "e.g. requests:^2.10.0 or requests=2.11.1."
+            ),
             flag=False,
             multiple=True,
         ),
         option(
             "dev-dependency",
             None,
-            "Package to require for development, with an optional version constraint, "
-            "e.g. requests:^2.10.0 or requests=2.11.1.",
+            (
+                "Package to require for development, with an optional version"
+                " constraint, e.g. requests:^2.10.0 or requests=2.11.1."
+            ),
             flag=False,
             multiple=True,
         ),
@@ -74,8 +76,9 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
         from poetry.core.pyproject.toml import PyProjectTOML
         from poetry.core.vcs.git import GitConfig
 
+        from poetry.config.config import Config
         from poetry.layouts import layout
-        from poetry.utils.env import SystemEnv
+        from poetry.utils.env import EnvManager
 
         project_path = Path.cwd()
 
@@ -157,10 +160,16 @@ The <c1>init</c1> command creates a basic <comment>pyproject.toml</> file in the
 
         python = self.option("python")
         if not python:
-            current_env = SystemEnv(Path(sys.executable))
-            default_python = "^" + ".".join(
-                str(v) for v in current_env.version_info[:2]
+            config = Config.create()
+            default_python = (
+                "^"
+                + EnvManager.get_python_version(
+                    precision=2,
+                    prefer_active_python=config.get("virtualenvs.prefer-active-python"),
+                    io=self.io,
+                ).to_string()
             )
+
             question = self.create_question(
                 f"Compatible Python versions [<comment>{default_python}</comment>]: ",
                 default=default_python,
@@ -231,8 +240,9 @@ You can specify a package in the following forms:
         )
 
         content = layout_.generate_poetry_content()
-        for section in content:
-            pyproject.data.append(section, content[section])
+        for section, item in content.items():
+            pyproject.data.append(section, item)
+
         if self.io.is_interactive():
             self.line("<info>Generated file</info>")
             self.line("")
@@ -318,8 +328,10 @@ You can specify a package in the following forms:
                     choices.append("")
 
                     package = self.choice(
-                        "\nEnter package # to add, or the complete package name if it"
-                        " is not listed",
+                        (
+                            "\nEnter package # to add, or the complete package name if"
+                            " it is not listed"
+                        ),
                         choices,
                         attempts=3,
                         default=len(choices) - 1,
@@ -413,7 +425,7 @@ You can specify a package in the following forms:
             # TODO: find similar
             raise ValueError(f"Could not find a matching version of package {name}")
 
-        return package.pretty_name, selector.find_recommended_require_version(package)
+        return package.pretty_name, f"^{package.version.to_string()}"
 
     def _parse_requirements(self, requirements: list[str]) -> list[dict[str, Any]]:
         from poetry.core.pyproject.exceptions import PyProjectException
