@@ -25,6 +25,7 @@ import packaging.tags
 import tomlkit
 import virtualenv
 
+from cleo.io.null_io import NullIO
 from cleo.io.outputs.output import Verbosity
 from packaging.tags import Tag
 from packaging.tags import interpreter_name
@@ -39,7 +40,6 @@ from virtualenv.seed.wheels.embed import get_embed_wheel
 from poetry.utils._compat import WINDOWS
 from poetry.utils._compat import decode
 from poetry.utils._compat import encode
-from poetry.utils._compat import list_to_shell_command
 from poetry.utils._compat import metadata
 from poetry.utils.helpers import get_real_windows_path
 from poetry.utils.helpers import is_dir_writable
@@ -58,7 +58,6 @@ if TYPE_CHECKING:
 
     from poetry.poetry import Poetry
 
-
 GET_SYS_TAGS = f"""
 import importlib.util
 import json
@@ -83,7 +82,6 @@ print(
 )
 """
 
-
 GET_ENVIRONMENT_INFO = """\
 import json
 import os
@@ -154,7 +152,6 @@ env = {
 print(json.dumps(env))
 """
 
-
 GET_BASE_PREFIX = """\
 import sys
 
@@ -173,8 +170,9 @@ print('.'.join([str(s) for s in sys.version_info[:3]]))
 """
 
 GET_PYTHON_VERSION_ONELINER = (
-    "\"import sys; print('.'.join([str(s) for s in sys.version_info[:3]]))\""
+    "import sys; print('.'.join([str(s) for s in sys.version_info[:3]]))"
 )
+GET_ENV_PATH_ONELINER = "import sys; print(sys.prefix)"
 
 GET_SYS_PATH = """\
 import json
@@ -327,8 +325,8 @@ class SitePackages:
         for distribution in self.distributions(
             name=distribution_name, writable_only=writable_only
         ):
-            assert distribution.files is not None
-            for file in distribution.files:
+            files = [] if distribution.files is None else distribution.files
+            for file in files:
                 if file.name.endswith(suffix):
                     yield Path(
                         distribution.locate_file(file),  # type: ignore[no-untyped-call]
@@ -340,8 +338,8 @@ class SitePackages:
         for distribution in self.distributions(
             name=distribution_name, writable_only=writable_only
         ):
-            assert distribution.files is not None
-            for file in distribution.files:
+            files = [] if distribution.files is None else distribution.files
+            for file in files:
                 if file.name == name:
                     yield Path(
                         distribution.locate_file(file),  # type: ignore[no-untyped-call]
@@ -371,8 +369,8 @@ class SitePackages:
         for distribution in self.distributions(
             name=distribution_name, writable_only=True
         ):
-            assert distribution.files is not None
-            for file in distribution.files:
+            files = [] if distribution.files is None else distribution.files
+            for file in files:
                 path = Path(
                     distribution.locate_file(file),  # type: ignore[no-untyped-call]
                 )
@@ -450,28 +448,31 @@ class SitePackages:
             if value[-1] is True
         ]
 
-    def __getattr__(self, item: str) -> Any:
-        try:
-            return super().__getattribute__(item)
-        except AttributeError:
-            return getattr(self.path, item)
-
 
 class EnvError(Exception):
     pass
 
 
+class IncorrectEnvError(EnvError):
+    def __init__(self, env_name: str) -> None:
+        message = f"Env {env_name} doesn't belong to this project."
+        super().__init__(message)
+
+
 class EnvCommandError(EnvError):
     def __init__(self, e: CalledProcessError, input: str | None = None) -> None:
         self.e = e
 
-        message = (
-            f"Command {e.cmd} errored with the following return code {e.returncode},"
-            f" and output: \n{decode(e.output)}"
-        )
+        message_parts = [
+            f"Command {e.cmd} errored with the following return code {e.returncode}"
+        ]
+        if e.output:
+            message_parts.append(f"Output:\n{decode(e.output)}")
+        if e.stderr:
+            message_parts.append(f"Error output:\n{decode(e.stderr)}")
         if input:
-            message += f"input was : {input}"
-        super().__init__(message)
+            message_parts.append(f"Input:\n{input}")
+        super().__init__("\n\n".join(message_parts))
 
 
 class NoCompatiblePythonVersionFound(EnvError):
@@ -514,17 +515,16 @@ class EnvManager:
 
     ENVS_FILE = "envs.toml"
 
-    def __init__(self, poetry: Poetry) -> None:
+    def __init__(self, poetry: Poetry, io: None | IO = None) -> None:
         self._poetry = poetry
+        self._io = io or NullIO()
 
-    def _full_python_path(self, python: str) -> str:
+    @staticmethod
+    def _full_python_path(python: str) -> str:
         try:
             executable = decode(
                 subprocess.check_output(
-                    list_to_shell_command(
-                        [python, "-c", '"import sys; print(sys.executable)"']
-                    ),
-                    shell=True,
+                    [python, "-c", "import sys; print(sys.executable)"],
                 ).strip()
             )
         except CalledProcessError as e:
@@ -532,26 +532,54 @@ class EnvManager:
 
         return executable
 
-    def _detect_active_python(self, io: IO) -> str | None:
+    @staticmethod
+    def _detect_active_python(io: None | IO = None) -> str | None:
+        io = io or NullIO()
         executable = None
 
         try:
-            io.write_line(
-                "Trying to detect current active python executable as specified in the"
-                " config.",
+            io.write_error_line(
+                (
+                    "Trying to detect current active python executable as specified in"
+                    " the config."
+                ),
                 verbosity=Verbosity.VERBOSE,
             )
-            executable = self._full_python_path("python")
-            io.write_line(f"Found: {executable}", verbosity=Verbosity.VERBOSE)
-        except CalledProcessError:
-            io.write_line(
-                "Unable to detect the current active python executable. Falling back to"
-                " default.",
+            executable = EnvManager._full_python_path("python")
+            io.write_error_line(f"Found: {executable}", verbosity=Verbosity.VERBOSE)
+        except EnvCommandError:
+            io.write_error_line(
+                (
+                    "Unable to detect the current active python executable. Falling"
+                    " back to default."
+                ),
                 verbosity=Verbosity.VERBOSE,
             )
         return executable
 
-    def activate(self, python: str, io: IO) -> Env:
+    @staticmethod
+    def get_python_version(
+        precision: int = 3,
+        prefer_active_python: bool = False,
+        io: None | IO = None,
+    ) -> Version:
+        version = ".".join(str(v) for v in sys.version_info[:precision])
+
+        if prefer_active_python:
+            executable = EnvManager._detect_active_python(io)
+
+            if executable:
+                python_patch = decode(
+                    subprocess.check_output(
+                        [executable, "-c", GET_PYTHON_VERSION_ONELINER],
+                    ).strip()
+                )
+
+                version = ".".join(str(v) for v in python_patch.split(".")[:precision])
+
+        return Version.parse(version)
+
+    def activate(self, python: str) -> Env:
         venv_path = self._poetry.config.virtualenvs_path
         cwd = self._poetry.file.parent
 
@@ -571,8 +599,7 @@ class EnvManager:
         try:
             python_version_string = decode(
                 subprocess.check_output(
-                    list_to_shell_command([python, "-c", GET_PYTHON_VERSION_ONELINER]),
-                    shell=True,
+                    [python, "-c", GET_PYTHON_VERSION_ONELINER],
                 )
             )
         except CalledProcessError as e:
@@ -597,7 +624,7 @@ class EnvManager:
                 if patch != current_patch:
                     create = True
 
-            self.create_venv(io, executable=python, force=create)
+            self.create_venv(executable=python, force=create)
 
             return self.get(reload=True)
 
@@ -631,7 +658,7 @@ class EnvManager:
                 if patch != current_patch:
                     create = True
 
-            self.create_venv(io, executable=python, force=create)
+            self.create_venv(executable=python, force=create)
 
         # Activate
         envs[base_env_name] = {"minor": minor, "patch": patch}
@@ -639,7 +666,7 @@ class EnvManager:
 
         return self.get(reload=True)
 
-    def deactivate(self, io: IO) -> None:
+    def deactivate(self) -> None:
         venv_path = self._poetry.config.virtualenvs_path
         name = self.generate_env_name(
             self._poetry.package.name, str(self._poetry.file.parent)
@@ -651,7 +678,9 @@ class EnvManager:
             env = envs.get(name)
             if env is not None:
                 venv = venv_path / f"{name}-py{env['minor']}"
-                io.write_line(f"Deactivating virtualenv: <comment>{venv}</comment>")
+                self._io.write_error_line(
+                    f"Deactivating virtualenv: <comment>{venv}</comment>"
+                )
                 del envs[name]
 
                 envs_file.write(envs)
@@ -660,7 +689,12 @@ class EnvManager:
         if self._env is not None and not reload:
             return self._env
 
-        python_minor = ".".join([str(v) for v in sys.version_info[:2]])
+        prefer_active_python = self._poetry.config.get(
+            "virtualenvs.prefer-active-python"
+        )
+        python_minor = self.get_python_version(
+            precision=2, prefer_active_python=prefer_active_python, io=self._io
+        ).to_string()
 
         venv_path = self._poetry.config.virtualenvs_path
 
@@ -731,13 +765,22 @@ class EnvManager:
 
         venv = self._poetry.file.parent / ".venv"
         if (
-            self._poetry.config.get("virtualenvs.in-project")
+            self._poetry.config.get("virtualenvs.in-project") is not False
             and venv.exists()
             and venv.is_dir()
         ):
             env_list.insert(0, VirtualEnv(venv))
         return env_list
 
+    @staticmethod
+    def check_env_is_for_current_project(env: str, base_env_name: str) -> bool:
+        """
+        Check if env name starts with projects name.
+
+        This is done to prevent action on other project's envs.
+        """
+        return env.startswith(base_env_name)
+
     def remove(self, python: str) -> Env:
         venv_path = self._poetry.config.virtualenvs_path
 
@@ -745,7 +788,22 @@ class EnvManager:
         envs_file = TOMLFile(venv_path / self.ENVS_FILE)
         base_env_name = self.generate_env_name(self._poetry.package.name, str(cwd))
 
-        if python.startswith(base_env_name):
+        python_path = Path(python)
+        if python_path.is_file():
+            # Validate env name if provided env is a full path to python
+            try:
+                env_dir = decode(
+                    subprocess.check_output(
+                        [python, "-c", GET_ENV_PATH_ONELINER],
+                    )
+                ).strip("\n")
+                env_name = Path(env_dir).name
+                if not self.check_env_is_for_current_project(env_name, base_env_name):
+                    raise IncorrectEnvError(env_name)
+            except CalledProcessError as e:
+                raise EnvCommandError(e)
+
+        if self.check_env_is_for_current_project(python, base_env_name):
             venvs = self.list()
             for venv in venvs:
                 if venv.path.name == python:
@@ -776,6 +834,12 @@ class EnvManager:
             raise ValueError(
                 f'<warning>Environment "{python}" does not exist.</warning>'
             )
+        else:
+            venv_path = self._poetry.config.virtualenvs_path
+            # Get all the poetry envs, even for other projects
+            env_names = [Path(p).name for p in sorted(venv_path.glob("*-*-py*"))]
+            if python in env_names:
+                raise IncorrectEnvError(python)
 
         try:
             python_version = Version.parse(python)
@@ -789,8 +853,7 @@ class EnvManager:
         try:
             python_version_string = decode(
                 subprocess.check_output(
-                    list_to_shell_command([python, "-c", GET_PYTHON_VERSION_ONELINER]),
-                    shell=True,
+                    [python, "-c", GET_PYTHON_VERSION_ONELINER],
                 )
             )
         except CalledProcessError as e:
@@ -821,7 +884,6 @@ class EnvManager:
 
     def create_venv(
         self,
-        io: IO,
         name: str | None = None,
         executable: str | None = None,
         force: bool = False,
@@ -854,7 +916,7 @@ class EnvManager:
         venv_prompt = self._poetry.config.get("virtualenvs.prompt")
 
         if not executable and prefer_active_python:
-            executable = self._detect_active_python(io)
+            executable = self._detect_active_python()
 
         venv_path = cwd / ".venv" if root_venv else self._poetry.config.virtualenvs_path
         if not name:
@@ -866,10 +928,7 @@ class EnvManager:
         if executable:
             python_patch = decode(
                 subprocess.check_output(
-                    list_to_shell_command(
-                        [executable, "-c", GET_PYTHON_VERSION_ONELINER]
-                    ),
-                    shell=True,
+                    [executable, "-c", GET_PYTHON_VERSION_ONELINER],
                 ).strip()
             )
             python_minor = ".".join(python_patch.split(".")[:2])
@@ -887,7 +946,7 @@ class EnvManager:
                     self._poetry.package.python_versions, python_patch
                 )
 
-            io.write_error_line(
+            self._io.write_error_line(
                 f"<warning>The currently activated Python version {python_patch} is not"
                 f" supported by the project ({self._poetry.package.python_versions}).\n"
                 "Trying to find and use a compatible version.</warning> "
@@ -910,17 +969,14 @@ class EnvManager:
 
                 python = "python" + python_to_try
 
-                if io.is_debug():
-                    io.write_line(f"<debug>Trying {python}</debug>")
+                if self._io.is_debug():
+                    self._io.write_error_line(f"<debug>Trying {python}</debug>")
 
                 try:
                     python_patch = decode(
                         subprocess.check_output(
-                            list_to_shell_command(
-                                [python, "-c", GET_PYTHON_VERSION_ONELINER]
-                            ),
+                            [python, "-c", GET_PYTHON_VERSION_ONELINER],
                             stderr=subprocess.STDOUT,
-                            shell=True,
                         ).strip()
                     )
                 except CalledProcessError:
@@ -930,8 +986,10 @@ class EnvManager:
                     continue
 
                 if supported_python.allows(Version.parse(python_patch)):
-                    io.write_line(f"Using <c1>{python}</c1> ({python_patch})")
-                    executable = python
+                    self._io.write_error_line(
+                        f"Using <c1>{python}</c1> ({python_patch})"
+                    )
+                    executable = self._full_python_path(python)
                     python_minor = ".".join(python_patch.split(".")[:2])
                     break
 
@@ -955,7 +1013,7 @@ class EnvManager:
 
         if not venv.exists():
             if create_venv is False:
-                io.write_line(
+                self._io.write_error_line(
                     "<fg=black;bg=yellow>"
                     "Skipping virtualenv creation, "
                     "as specified in config file."
@@ -964,7 +1022,7 @@ class EnvManager:
 
                 return self.get_system_env()
 
-            io.write_line(
+            self._io.write_error_line(
                 f"Creating virtualenv <c1>{name}</> in"
                 f" {venv_path if not WINDOWS else get_real_windows_path(venv_path)!s}"
             )
@@ -972,15 +1030,17 @@ class EnvManager:
             create_venv = False
             if force:
                 if not env.is_sane():
-                    io.write_error_line(
+                    self._io.write_error_line(
                         f"<warning>The virtual environment found in {env.path} seems to"
                         " be broken.</warning>"
                     )
-                io.write_line(f"Recreating virtualenv <c1>{name}</> in {venv!s}")
+                self._io.write_error_line(
+                    f"Recreating virtualenv <c1>{name}</> in {venv!s}"
+                )
                 self.remove_venv(venv)
                 create_venv = True
-            elif io.is_very_verbose():
-                io.write_line(f"Virtualenv <c1>{name}</> already exists.")
+            elif self._io.is_very_verbose():
+                self._io.write_error_line(f"Virtualenv <c1>{name}</> already exists.")
 
         if create_venv:
             self.build_venv(
@@ -1362,6 +1422,16 @@ class Env:
         if self._paths is None:
             self._paths = self.get_paths()
 
+            if self.is_venv():
+                # We copy pip's logic here for the `include` path
+                self._paths["include"] = str(
+                    self.path.joinpath(
+                        "include",
+                        "site",
+                        f"python{self.version_info[0]}.{self.version_info[1]}",
+                    )
+                )
+
         return self._paths
 
     @property
@@ -1435,7 +1505,16 @@ class Env:
         return self._run(cmd, **kwargs)
 
     def run_python_script(self, content: str, **kwargs: Any) -> int | str:
-        return self.run(self._executable, "-W", "ignore", "-", input_=content, **kwargs)
+        return self.run(
+            self._executable,
+            "-I",
+            "-W",
+            "ignore",
+            "-",
+            input_=content,
+            stderr=subprocess.PIPE,
+            **kwargs,
+        )
 
     def _run(self, cmd: list[str], **kwargs: Any) -> int | str:
         """
@@ -1444,34 +1523,24 @@ class Env:
         call = kwargs.pop("call", False)
         input_ = kwargs.pop("input_", None)
         env = kwargs.pop("env", dict(os.environ))
+        stderr = kwargs.pop("stderr", subprocess.STDOUT)
 
         try:
-            if self._is_windows:
-                kwargs["shell"] = True
-
-            command: str | list[str]
-            if kwargs.get("shell", False):
-                command = list_to_shell_command(cmd)
-            else:
-                command = cmd
-
             if input_:
                 output = subprocess.run(
-                    command,
+                    cmd,
                     stdout=subprocess.PIPE,
-                    stderr=subprocess.STDOUT,
+                    stderr=stderr,
                     input=encode(input_),
                     check=True,
                     **kwargs,
                 ).stdout
             elif call:
                 return subprocess.call(
-                    command, stderr=subprocess.STDOUT, env=env, **kwargs
+                    cmd, stdout=subprocess.PIPE, stderr=stderr, env=env, **kwargs
                 )
             else:
-                output = subprocess.check_output(
-                    command, stderr=subprocess.STDOUT, env=env, **kwargs
-                )
+                output = subprocess.check_output(cmd, stderr=stderr, env=env, **kwargs)
         except CalledProcessError as e:
             raise EnvCommandError(e, input=input_)
 
@@ -1485,7 +1554,7 @@ class Env:
             return os.execvpe(command[0], command, env=env)
 
         kwargs["shell"] = True
-        exe = subprocess.Popen([command[0]] + command[1:], env=env, **kwargs)
+        exe = subprocess.Popen(command, env=env, **kwargs)
         exe.communicate()
         return exe.returncode
 
@@ -1824,7 +1893,7 @@ class GenericEnv(VirtualEnv):
         if not self._is_windows:
             return os.execvpe(command[0], command, env=env)
 
-        exe = subprocess.Popen([command[0]] + command[1:], env=env, **kwargs)
+        exe = subprocess.Popen(command, env=env, **kwargs)
         exe.communicate()
 
         return exe.returncode
@@ -1848,6 +1917,17 @@ class NullEnv(SystemEnv):
         self._execute = execute
         self.executed: list[list[str]] = []
 
+    @property
+    def paths(self) -> dict[str, str]:
+        if self._paths is None:
+            self._paths = self.get_paths()
+            self._paths["platlib"] = str(self._path / "platlib")
+            self._paths["purelib"] = str(self._path / "purelib")
+            self._paths["scripts"] = str(self._path / "scripts")
+            self._paths["data"] = str(self._path / "data")
+
+        return self._paths
+
     def _run(self, cmd: list[str], **kwargs: Any) -> int | str:
         self.executed.append(cmd)
 
@@ -1901,14 +1981,14 @@ def build_environment(
 
             if io:
                 if not overwrite:
-                    io.write_line("")
+                    io.write_error_line("")
 
                 requires = [
                     f"<c1>{requirement}</c1>"
                     for requirement in poetry.pyproject.build_system.requires
                 ]
 
-                io.overwrite(
+                io.overwrite_error(
                     "<b>Preparing</b> build environment with build-system requirements"
                     f" {', '.join(requires)}"
                 )
@@ -1917,12 +1997,13 @@ def build_environment(
                 "install",
                 "--disable-pip-version-check",
                 "--ignore-installed",
+                "--no-input",
                 *poetry.pyproject.build_system.requires,
             )
 
             if overwrite:
                 assert io is not None
-                io.write_line("")
+                io.write_error_line("")
 
             yield venv
     else:
